@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -187,7 +187,7 @@ typedef enum {
 
 GRN_API const char *
 grn_log_level_to_string(grn_log_level level);
-GRN_API grn_bool
+GRN_API bool
 grn_log_level_parse(const char *string, grn_log_level *level);
 
 /* query log flags */
@@ -320,10 +320,18 @@ GRN_API grn_bool
 grn_ctx_get_force_match_escalation(grn_ctx *ctx);
 GRN_API grn_rc
 grn_ctx_set_force_match_escalation(grn_ctx *ctx, grn_bool force);
+GRN_API int32_t
+grn_ctx_get_n_workers(grn_ctx *ctx);
+GRN_API grn_rc
+grn_ctx_set_n_workers(grn_ctx *ctx, int32_t n_workers);
 GRN_API int64_t
 grn_get_default_match_escalation_threshold(void);
 GRN_API grn_rc
 grn_set_default_match_escalation_threshold(int64_t threshold);
+GRN_API int32_t
+grn_get_default_n_workers(void);
+GRN_API grn_rc
+grn_set_default_n_workers(int32_t n_workers);
 GRN_API bool
 grn_is_back_trace_enable(void);
 GRN_API grn_rc
@@ -591,7 +599,7 @@ typedef enum {
 
 GRN_API grn_obj *
 grn_ctx_at(grn_ctx *ctx, grn_id id);
-GRN_API grn_bool
+GRN_API bool
 grn_ctx_is_opened(grn_ctx *ctx, grn_id id);
 
 GRN_API grn_rc
@@ -989,6 +997,8 @@ struct _grn_search_optarg {
   grn_obj *query_options;
   grn_obj *max_element_intervals;
   int *min_interval;
+  int32_t *start_position;
+  grn_id *query_domain;
 };
 
 GRN_API grn_rc
@@ -1240,7 +1250,7 @@ struct _grn_logger {
   void (*fin)(grn_ctx *ctx, void *user_data);
 };
 
-GRN_API grn_bool
+GRN_API bool
 grn_log_flags_parse(const char *string, int string_size, int *flags);
 
 GRN_API grn_rc
@@ -1295,10 +1305,10 @@ grn_logger_putv(grn_ctx *ctx,
 GRN_API void
 grn_logger_reopen(grn_ctx *ctx);
 
-GRN_API grn_bool
+GRN_API bool
 grn_logger_pass(grn_ctx *ctx, grn_log_level level);
 
-GRN_API grn_bool
+GRN_API bool
 grn_logger_is_default_logger(grn_ctx *ctx);
 
 #ifndef GRN_LOG_DEFAULT_LEVEL
@@ -1375,7 +1385,7 @@ struct _grn_query_logger {
   void (*fin)(grn_ctx *ctx, void *user_data);
 };
 
-GRN_API grn_bool
+GRN_API bool
 grn_query_log_flags_parse(const char *string,
                           int string_size,
                           unsigned int *flags);
@@ -1398,7 +1408,7 @@ grn_query_logger_put(
 GRN_API void
 grn_query_logger_reopen(grn_ctx *ctx);
 
-GRN_API grn_bool
+GRN_API bool
 grn_query_logger_pass(grn_ctx *ctx, unsigned int flag);
 
 GRN_API void
@@ -1732,6 +1742,12 @@ grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_GEO_DEGREE2MSEC(degree)                                            \
   ((int32_t)((degree)*3600 * 1000 + ((degree) > 0 ? 0.5 : -0.5)))
 #define GRN_GEO_MSEC2DEGREE(msec) ((((int32_t)(msec)) / 3600.0) * 0.001)
+#define GRN_GEO_RESOLUTION        3600000
+#define GRN_GEO_M_PI              3.14159265358979323846
+#define GRN_GEO_RADIAN2MSEC(radian)                                            \
+  ((int32_t)(((GRN_GEO_RESOLUTION * 180) / GRN_GEO_M_PI) * (radian)))
+#define GRN_GEO_MSEC2RADIAN(msec)                                              \
+  ((GRN_GEO_M_PI / (GRN_GEO_RESOLUTION * 180)) * (msec))
 
 #define GRN_GEO_POINT_SET(ctx, obj, _latitude, _longitude)                     \
   do {                                                                         \
@@ -1869,27 +1885,34 @@ grn_ctx_recv_handler_set(grn_ctx *,
                         sizeof(grn_obj *));                                    \
   } while (0)
 
-#define GRN_BOOL_VALUE(obj)     (*((bool *)GRN_BULK_HEAD(obj)))
-#define GRN_INT8_VALUE(obj)     (*((int8_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT8_VALUE(obj)    (*((uint8_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT16_VALUE(obj)    (*((int16_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT16_VALUE(obj)   (*((uint16_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT32_VALUE(obj)    (*((int32_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT32_VALUE(obj)   (*((uint32_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT64_VALUE(obj)    (*((int64_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT64_VALUE(obj)   (*((uint64_t *)GRN_BULK_HEAD(obj)))
-#define GRN_BFLOAT16_VALUE(obj) (*((grn_bfloat16 *)GRN_BULK_HEAD(obj)))
-#define GRN_FLOAT32_VALUE(obj)  (*((float *)GRN_BULK_HEAD(obj)))
-#define GRN_FLOAT_VALUE(obj)    (*((double *)GRN_BULK_HEAD(obj)))
-#define GRN_TIME_VALUE          GRN_INT64_VALUE
-#define GRN_RECORD_VALUE(obj)   (*((grn_id *)GRN_BULK_HEAD(obj)))
-#define GRN_PTR_VALUE(obj)      (*((grn_obj **)GRN_BULK_HEAD(obj)))
+#define GRN_BOOL_VALUE(obj)          (*((bool *)GRN_BULK_HEAD(obj)))
+#define GRN_INT8_VALUE(obj)          (*((int8_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT8_VALUE(obj)         (*((uint8_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT16_VALUE(obj)         (*((int16_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT16_VALUE(obj)        (*((uint16_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT32_VALUE(obj)         (*((int32_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT32_VALUE(obj)        (*((uint32_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT64_VALUE(obj)         (*((int64_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT64_VALUE(obj)        (*((uint64_t *)GRN_BULK_HEAD(obj)))
+#define GRN_BFLOAT16_VALUE(obj)      (*((grn_bfloat16 *)GRN_BULK_HEAD(obj)))
+#define GRN_FLOAT32_VALUE(obj)       (*((float *)GRN_BULK_HEAD(obj)))
+#define GRN_FLOAT_VALUE(obj)         (*((double *)GRN_BULK_HEAD(obj)))
+#define GRN_TIME_VALUE               GRN_INT64_VALUE
+#define GRN_RECORD_VALUE(obj)        (*((grn_id *)GRN_BULK_HEAD(obj)))
+#define GRN_PTR_VALUE(obj)           (*((grn_obj **)GRN_BULK_HEAD(obj)))
+#define GRN_GEO_POINT_VALUE_RAW(obj) (grn_geo_point *)GRN_BULK_HEAD(obj)
 #define GRN_GEO_POINT_VALUE(obj, _latitude, _longitude)                        \
   do {                                                                         \
-    grn_geo_point *_val = (grn_geo_point *)GRN_BULK_HEAD(obj);                 \
+    grn_geo_point *_val = GRN_GEO_POINT_VALUE_RAW(obj);                        \
     _latitude = _val->latitude;                                                \
     _longitude = _val->longitude;                                              \
   } while (0)
+#define GRN_GEO_POINT_VALUE_RADIAN(obj, _latitude, _longitude)                 \
+  do {                                                                         \
+    grn_geo_point *_val = GRN_GEO_POINT_VALUE_RAW(obj);                        \
+    _latitude = GRN_GEO_MSEC2RADIAN(_val->latitude);                           \
+    _longitude = GRN_GEO_MSEC2RADIAN(_val->longitude);                         \
+  } while (0)
 
 #define GRN_BOOL_VALUE_AT(obj, offset) (((bool *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_INT8_VALUE_AT(obj, offset) (((int8_t *)GRN_BULK_HEAD(obj))[offset])
