@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2009-2016  Brazil
-  Copyright (C) 2020-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2020-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -93,6 +93,36 @@ grn_rc grn_ctx_release_child(grn_ctx *ctx, grn_ctx *child_ctx);
 
 void grn_ctx_call_progress_callback(grn_ctx *ctx, grn_progress *progress);
 
+void grn_ctx_trace_log_enable(grn_ctx *ctx);
+void grn_ctx_trace_log_disable(grn_ctx *ctx);
+void grn_ctx_trace_log_dump(grn_ctx *ctx,
+                            grn_ctx *dump_ctx,
+                            grn_obj *output);
+size_t grn_ctx_trace_log_restore(grn_ctx *ctx,
+                                 const uint8_t *value,
+                                 size_t value_size);
+uint16_t grn_ctx_trace_log_get_current_depth(grn_ctx *ctx);
+void grn_ctx_trace_log_set_current_depth(grn_ctx *ctx, uint16_t depth);
+void grn_ctx_trace_log_push(grn_ctx *ctx);
+void grn_ctx_trace_log_pop(grn_ctx *ctx);
+void grn_ctx_trace_log_emit_string(grn_ctx *ctx,
+                                   const char *name,
+                                   const char *value,
+                                   size_t value_length);
+void grn_ctx_trace_log_emit_cstring(grn_ctx *ctx,
+                                    const char *name,
+                                    const char *value);
+void grn_ctx_trace_log_emit_uint32(grn_ctx *ctx,
+                                   const char *name,
+                                   uint32_t n);
+void grn_ctx_trace_log_emit_record_key(grn_ctx *ctx,
+                                       const char *name,
+                                       grn_obj *table,
+                                       grn_id id);
+void grn_ctx_trace_log_emit_object(grn_ctx *ctx,
+                                   const char *name,
+                                   grn_obj *object);
+
 grn_content_type grn_get_ctype(grn_obj *var);
 grn_content_type grn_content_type_parse(grn_ctx *ctx,
                                         grn_obj *var,
