@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -26,12 +26,12 @@
 #include "grn_arrow.h"
 
 #ifdef GRN_WITH_MRUBY
-# include <mruby.h>
+#  include <mruby.h>
 #endif
 
 #ifdef GRN_WITH_LUAJIT
-# include <lauxlib.h>
-# include <lualib.h>
+#  include <lauxlib.h>
+#  include <lualib.h>
 #endif
 
 #ifdef __cplusplus
@@ -40,19 +40,18 @@ extern "C" {
 
 /**** grn_expr ****/
 
-#define GRN_EXPR_MISSING_NAME          "expr_missing"
+#define GRN_EXPR_MISSING_NAME "expr_missing"
 
 /**** grn_ctx_impl ****/
 
-#define GRN_CTX_INITED    0x00
-#define GRN_CTX_QUITTING  0x0f
+#define GRN_CTX_INITED     0x00
+#define GRN_CTX_QUITTING   0x0f
 
 #define GRN_CTX_N_SEGMENTS 512
 
-#ifdef USE_MEMORY_DEBUG
+#ifdef GRN_WITH_MEMORY_DEBUG
 typedef struct _grn_alloc_info grn_alloc_info;
-struct _grn_alloc_info
-{
+struct _grn_alloc_info {
   void *address;
   int freed;
   size_t size;
@@ -105,7 +104,7 @@ struct _grn_ctx_impl {
   grn_critical_section lock;
   grn_io_mapinfo segs[GRN_CTX_N_SEGMENTS];
 
-#ifdef USE_MEMORY_DEBUG
+#ifdef GRN_WITH_MEMORY_DEBUG
   /* memory debug portion */
   grn_alloc_info *alloc_info;
 #endif
@@ -151,12 +150,18 @@ struct _grn_ctx_impl {
     grn_arrow_stream_writer *arrow_stream_writer;
   } output;
 
+  struct {
+    int32_t n_workers;
+    void *task_executor;
+  } parallel;
+
   struct {
     int flags;
     grn_command_version version;
     struct {
       grn_obj *command;
       grn_command_version version;
+      int32_t n_workers;
     } keep;
   } command;
 
@@ -168,12 +173,12 @@ struct _grn_ctx_impl {
   grn_proc_func *finalizer;
 
   grn_obj *db;
-  grn_array *values;        /* temporary objects */
+  grn_array *values; /* temporary objects */
   grn_pat *temporary_columns;
   grn_options *temporary_options;
   grn_critical_section columns_cache_lock;
   grn_hash *columns_cache;
-  grn_hash *ios;        /* IOs */
+  grn_hash *ios; /* IOs */
   grn_com *com;
   unsigned int com_status;
 
@@ -182,6 +187,10 @@ struct _grn_ctx_impl {
   char previous_errbuf[GRN_CTX_MSGSIZE];
   unsigned int n_same_error_messages;
 
+  struct {
+    grn_obj start_times;
+  } slow_log;
+
   grn_mrb_data mrb;
   grn_lua_data lua;
 
@@ -207,12 +216,38 @@ struct _grn_ctx_impl {
     grn_progress_callback_func callback;
     void *user_data;
   } progress;
+
+  struct {
+    uint64_t start_time;
+    uint16_t current_depth;
+    grn_obj depths;
+    grn_obj sequence_stack;
+    grn_obj sequences;
+    grn_obj names;
+    grn_obj values;
+    /* in nano seconds */
+    grn_obj elapsed_times;
+  } trace_log;
 };
 
 #define GRN_CTX_GET_WAL_ROLE(ctx) ((ctx)->impl->wal.role)
 
-void grn_ctx_impl_columns_cache_delete(grn_ctx *ctx, grn_id table_id);
-void grn_ctx_impl_columns_cache_clear(grn_ctx *ctx);
+static inline bool
+grn_ctx_trace_log_is_enabled(grn_ctx *ctx)
+{
+  if (!ctx) {
+    return false;
+  }
+  if (!ctx->impl) {
+    return false;
+  }
+  return ctx->impl->trace_log.start_time != 0;
+}
+
+void
+grn_ctx_impl_columns_cache_delete(grn_ctx *ctx, grn_id table_id);
+void
+grn_ctx_impl_columns_cache_clear(grn_ctx *ctx);
 
 #ifdef __cplusplus
 }
