@@ -1,6 +1,5 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2016 Brazil
+  Copyright(C) 2013-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -34,6 +33,7 @@ void grn_mrb_init_from_env(void);
 #ifdef GRN_WITH_MRUBY
 GRN_API mrb_value grn_mrb_load(grn_ctx *ctx, const char *path);
 GRN_API const char *grn_mrb_get_system_ruby_scripts_dir(grn_ctx *ctx);
+grn_bool grn_mrb_is_order_by_estimated_size_enabled(void);
 #endif
 
 #ifdef __cplusplus
