@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2016  Brazil
-  Copyright(C) 2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2009-2016  Brazil
+  Copyright (C) 2021-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,18 +29,16 @@
 #define MAX_SNIP_RESULT_COUNT      16U
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
-#define SNIPCOND_NONSTOP 0
-#define SNIPCOND_STOP    1
-#define SNIPCOND_ACROSS  2
+#define SNIPCOND_NONSTOP          0
+#define SNIPCOND_STOP             1
+#define SNIPCOND_ACROSS           2
 
 #define GRN_QUERY_SCAN_ALLOCCONDS 0x0002
 
-typedef struct _snip_cond
-{
+typedef struct _snip_cond {
   /* initial parameters */
   const char *opentag;
   const char *closetag;
@@ -49,16 +47,16 @@ typedef struct _snip_cond
   grn_obj *keyword;
 
   /* Tuned BM pre */
-  size_t bmBc[ASIZE];
-  size_t shift;
+  ptrdiff_t bmBc[ASIZE];
+  ptrdiff_t shift;
 
   /* Tuned BM temporal result */
-  size_t found;
-  size_t last_found;
+  ptrdiff_t found;
+  ptrdiff_t last_found;
   size_t last_offset;
   size_t start_offset;
   size_t end_offset;
-  size_t found_alpha_head;
+  ptrdiff_t found_alpha_head;
 
   /* search result */
   int count;
@@ -67,15 +65,13 @@ typedef struct _snip_cond
   int_least8_t stopflag;
 } snip_cond;
 
-typedef struct
-{
+typedef struct {
   size_t start_offset;
   size_t end_offset;
   snip_cond *cond;
 } _snip_tag_result;
 
-typedef struct
-{
+typedef struct {
   size_t start_offset;
   size_t end_offset;
   unsigned int first_tag_result_idx;
@@ -83,8 +79,7 @@ typedef struct
   unsigned int tag_count;
 } _snip_result;
 
-typedef struct _grn_snip
-{
+typedef struct _grn_snip {
   grn_db_obj obj;
   grn_encoding encoding;
   int flags;
@@ -113,6 +108,8 @@ typedef struct _grn_snip
   size_t max_tagged_len;
 
   grn_obj *normalizer;
+  grn_obj *lexicon;
+  grn_obj normalizers;
 
   char *delimiter_pattern;
   size_t delimiter_pattern_length;
@@ -121,12 +118,12 @@ typedef struct _grn_snip
 #endif
 } grn_snip;
 
-grn_rc grn_snip_close(grn_ctx *ctx, grn_snip *snip);
-grn_rc grn_snip_cond_init(grn_ctx *ctx, snip_cond *sc, const char *keyword, unsigned int keyword_len,
-                          grn_encoding enc, grn_obj *normalizer, int flags);
-void grn_snip_cond_reinit(snip_cond *cond);
-grn_rc grn_snip_cond_close(grn_ctx *ctx, snip_cond *cond);
-void grn_bm_tunedbm(grn_ctx *ctx, snip_cond *cond, grn_obj *string, int flags);
+grn_rc
+grn_snip_close(grn_ctx *ctx, grn_snip *snip);
+grn_rc
+grn_snip_cond_close(grn_ctx *ctx, snip_cond *cond);
+void
+grn_bm_tunedbm(grn_ctx *ctx, snip_cond *cond, grn_obj *string, int flags);
 
 #ifdef __cplusplus
 }
