@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2010-2018 Brazil
 
@@ -42,15 +41,12 @@ grn_rc grn_uvector_record_inspect_without_columns(grn_ctx *ctx,
  * with a string that makes the filename unique.
  */
 int grn_mkstemp(char *path_template);
-grn_bool grn_path_exist(const char *path);
+bool grn_path_exist(const char *path);
 
 int grn_tokenize(const char *str, size_t str_len,
                  const char **tokbuf, int buf_size,
                  const char **rest);
 
-grn_obj *
-grn_inspect_key(grn_ctx *ctx, grn_obj *buffer,
-                grn_obj *table, const void *key, unsigned int key_size);
 
 #ifdef __cplusplus
 }
