@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2009-2018  Brazil
+  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -66,16 +66,15 @@ command_table_create_parse_flags(grn_ctx *ctx,
   return flags;
 }
 
-grn_bool
+bool
 grn_proc_table_set_token_filters(grn_ctx *ctx,
                                  grn_obj *table,
                                  grn_raw_string *token_filters_raw)
 {
-  grn_bool succeeded = GRN_FALSE;
   grn_obj token_filters;
 
   if (token_filters_raw->length == 0) {
-    return GRN_TRUE;
+    return true;
   }
 
   GRN_TEXT_INIT(&token_filters, GRN_OBJ_DO_SHALLOW_COPY);
@@ -86,7 +85,7 @@ grn_proc_table_set_token_filters(grn_ctx *ctx,
   grn_obj_set_info(ctx, table, GRN_INFO_TOKEN_FILTERS, &token_filters);
   GRN_OBJ_FIN(ctx, &token_filters);
 
-  return succeeded;
+  return false;
 }
 
 static grn_obj *
@@ -131,9 +130,9 @@ command_table_create(grn_ctx *ctx,
 
 #undef GET_VALUE
 
-  flags = grn_atoi(flags_raw.value,
-                   flags_raw.value + flags_raw.length,
-                   &rest);
+  flags = (grn_table_flags)grn_atoi(flags_raw.value,
+                                    flags_raw.value + flags_raw.length,
+                                    &rest);
   if (flags_raw.value == rest) {
     flags = command_table_create_parse_flags(ctx,
                                              flags_raw.value,
@@ -152,7 +151,7 @@ command_table_create(grn_ctx *ctx,
     if (key_type_raw.length > 0) {
       key_type = grn_ctx_get(ctx,
                              key_type_raw.value,
-                             key_type_raw.length);
+                             (int)(key_type_raw.length));
       if (!key_type) {
         GRN_PLUGIN_ERROR(ctx,
                          GRN_INVALID_ARGUMENT,
@@ -169,7 +168,7 @@ command_table_create(grn_ctx *ctx,
     if (value_type_raw.length > 0) {
       value_type = grn_ctx_get(ctx,
                                value_type_raw.value,
-                               value_type_raw.length);
+                               (int)(value_type_raw.length));
       if (!value_type) {
         GRN_PLUGIN_ERROR(ctx,
                          GRN_INVALID_ARGUMENT,
@@ -191,7 +190,7 @@ command_table_create(grn_ctx *ctx,
     flags |= GRN_OBJ_PERSISTENT;
     table = grn_table_create(ctx,
                              name_raw.value,
-                             name_raw.length,
+                             (unsigned int)(name_raw.length),
                              path,
                              flags,
                              key_type,
@@ -245,7 +244,7 @@ command_table_create(grn_ctx *ctx,
 
 exit :
   {
-    grn_bool success = (ctx->rc == GRN_SUCCESS);
+    bool success = (ctx->rc == GRN_SUCCESS);
     if (key_type) {
       grn_obj_unref(ctx, key_type);
     }
@@ -348,10 +347,7 @@ command_table_list(grn_ctx *ctx, int nargs, grn_obj **args,
 {
   grn_obj *db;
   grn_obj tables;
-  int n_top_level_elements;
   int n_elements_for_header = 1;
-  int n_tables;
-  int i;
 
   db = grn_ctx_db(ctx);
 
@@ -366,7 +362,7 @@ command_table_list(grn_ctx *ctx, int nargs, grn_obj **args,
     prefix = grn_plugin_proc_get_var(ctx, user_data, "prefix", -1);
     if (GRN_TEXT_LEN(prefix) > 0) {
       min = GRN_TEXT_VALUE(prefix);
-      min_size = GRN_TEXT_LEN(prefix);
+      min_size = (unsigned int)GRN_TEXT_LEN(prefix);
       flags |= GRN_CURSOR_PREFIX;
     }
     cursor = grn_table_cursor_open(ctx, db,
@@ -383,13 +379,13 @@ command_table_list(grn_ctx *ctx, int nargs, grn_obj **args,
       const char *name;
       void *key;
       int i, key_size;
-      grn_bool have_period = GRN_FALSE;
+      bool have_period = false;
 
       key_size = grn_table_cursor_get_key(ctx, cursor, &key);
       name = key;
       for (i = 0; i < key_size; i++) {
         if (name[i] == '.') {
-          have_period = GRN_TRUE;
+          have_period = true;
           break;
         }
       }
@@ -412,8 +408,8 @@ command_table_list(grn_ctx *ctx, int nargs, grn_obj **args,
     }
     grn_table_cursor_close(ctx, cursor);
   }
-  n_tables = GRN_BULK_VSIZE(&tables) / sizeof(grn_obj *);
-  n_top_level_elements = n_elements_for_header + n_tables;
+  size_t n_tables = GRN_BULK_VSIZE(&tables) / sizeof(grn_obj *);
+  int n_top_level_elements = n_elements_for_header + (int)n_tables;
   grn_ctx_output_array_open(ctx, "TABLE_LIST", n_top_level_elements);
 
   grn_ctx_output_array_open(ctx, "HEADER", 8);
@@ -451,6 +447,7 @@ command_table_list(grn_ctx *ctx, int nargs, grn_obj **args,
   grn_ctx_output_array_close(ctx);
   grn_ctx_output_array_close(ctx);
 
+  size_t i;
   for (i = 0; i < n_tables; i++) {
     grn_obj *table = GRN_PTR_VALUE_AT(&tables, i);
     output_table_info(ctx, table);
@@ -484,21 +481,21 @@ command_table_remove(grn_ctx *ctx,
 {
   grn_obj *name;
   grn_obj *table;
-  grn_bool dependent;
+  bool dependent;
 
   name = grn_plugin_proc_get_var(ctx, user_data, "name", -1);
   dependent = grn_plugin_proc_get_var_bool(ctx, user_data, "dependent", -1,
-                                           GRN_FALSE);
+                                           false);
   table = grn_ctx_get(ctx,
                       GRN_TEXT_VALUE(name),
-                      GRN_TEXT_LEN(name));
+                      (int)GRN_TEXT_LEN(name));
   if (!table) {
     GRN_PLUGIN_ERROR(ctx,
                      GRN_INVALID_ARGUMENT,
                      "[table][remove] table isn't found: <%.*s>",
                      (int)GRN_TEXT_LEN(name),
                      GRN_TEXT_VALUE(name));
-    grn_ctx_output_bool(ctx, GRN_FALSE);
+    grn_ctx_output_bool(ctx, false);
     return NULL;
   }
 
@@ -512,7 +509,7 @@ command_table_remove(grn_ctx *ctx,
                      (int)GRN_TEXT_LEN(name),
                      GRN_TEXT_VALUE(name),
                      type_name);
-    grn_ctx_output_bool(ctx, GRN_FALSE);
+    grn_ctx_output_bool(ctx, false);
     return NULL;
   }
 
@@ -557,7 +554,7 @@ command_table_rename(grn_ctx *ctx,
     GRN_PLUGIN_ERROR(ctx, rc, "[table][rename] table name isn't specified");
     goto exit;
   }
-  table = grn_ctx_get(ctx, GRN_TEXT_VALUE(name), GRN_TEXT_LEN(name));
+  table = grn_ctx_get(ctx, GRN_TEXT_VALUE(name), (int)GRN_TEXT_LEN(name));
   if (!table) {
     rc = GRN_INVALID_ARGUMENT;
     GRN_PLUGIN_ERROR(ctx,
@@ -578,7 +575,7 @@ command_table_rename(grn_ctx *ctx,
   }
   rc = grn_table_rename(ctx, table,
                         GRN_TEXT_VALUE(new_name),
-                        GRN_TEXT_LEN(new_name));
+                        (unsigned int)GRN_TEXT_LEN(new_name));
   if (rc != GRN_SUCCESS && ctx->rc == GRN_SUCCESS) {
     GRN_PLUGIN_ERROR(ctx,
                      rc,
@@ -683,7 +680,7 @@ command_table_create_similar(grn_ctx *ctx,
 
   table = grn_table_create_similar(ctx,
                                    GRN_TEXT_VALUE(name),
-                                   GRN_TEXT_LEN(name),
+                                   (uint32_t)GRN_TEXT_LEN(name),
                                    NULL,
                                    base_table);
 
